<?php

/**
 * If called from a cache, Drupal wasn't bootstrapped and the following aren't
 * defined, so we define them.
 */
if (!defined('QUOVA_AD_ALL')) {
  define('QUOVA_AD_ALL', 0);
  define('QUOVA_AD_SELECTED', 1);
  define('QUOVA_AD_NOT_SELECTED', 2);
}

/**
 * Cache filter
 */
function quova_ad_cache_filter($ads) {
  _debug_echo("quova_ad_cache: adserve_cache_filter");
  $geoip = adserve_cache('get_cache', 'geoip');
  $valid = array();
  foreach ($ads as $aid) {
    $targets = array('country', 'region', 'city');
    $display = TRUE;
    foreach ($targets as $target) {
      // stop processing if we already know we shouldn't display this ad
      if ($display !== TRUE) break;
      $format = isset($geoip[$target]['format'][$aid]) ? $geoip[$target]['format'][$aid] : QUOVA_AD_ALL;
      _debug_echo("quova_ad_cache: aid($aid) $target format($format)");

      foreach ($geoip[$aid][$target] as $code) {
        $display = quova_ad_display($target, $format, $code);
        // break out if we already know we shouldn't display this ad
        if ($format == QUOVA_AD_ALL) {
          _debug_echo("quova_ad_cache: $target: display aid($aid)");
          break;
        }
        else if ($display === FALSE) {
          _debug_echo("quova_ad_cache: do not display aid($aid)");
          break;
        }
        // break out if we already know we should display this ad
        else if ($format == QUOVA_AD_SELECTED && $display === TRUE) {
          _debug_echo("quova_ad_cache: $target: display aid($aid)");
          break;
        }
      }
    }

    if ($display === TRUE) {
      _debug_echo("quova_ad_cache: aid($aid) is valid");
      $valid[] = $aid;
    }
    else {
      _debug_echo("quova_ad_cache: aid($aid) is not valid");
    }
  }
  return $valid;
}

/**
 * Determine if the advertisement should be displayed.
 */
function quova_ad_display($target, $format, $code) {
  switch ($format) {
    // display to all
    case QUOVA_AD_ALL: {
      _debug_echo("quova_ad_cache: $format: all valid.");
      $display = TRUE;
      break;
    }

    // display to selected
    case QUOVA_AD_SELECTED: {
      $lookup = quova_ad_lookup($target);
      if (strcasecmp($code, $lookup) == 0) {
        _debug_echo("quova_ad_cache: $format:  current visitor from $code.");
        $display = TRUE;
        break;
      }
      else if ($lookup == 'UNKNOWN') {
        _debug_echo("quova_ad_cache: $format: current visitor from UNKOWN locale.");
        $display = FALSE;
      }
      else {
        if ($target == 'city' && strpos($code, '*') !== FALSE) {
          // city with wildcard
          if (quova_ad_search($code, $lookup)) {
            _debug_echo("quova_ad_cache: $format:  current visitor from $lookup, matches $code.");
            $display = TRUE;
            break;
          }
        }
        _debug_echo("quova_ad_cache: $format:  current visitor from $lookup not from $code.");
        $display = NULL;
      }
      break;
    }

    // display to not selected
    case QUOVA_AD_NOT_SELECTED: {
      $lookup = quova_ad_lookup($target);
      if (strcasecmp($code, $lookup) == 0) {
        _debug_echo("quova_ad_cache: current visitor from $code.");
        $display = FALSE;
      }
      else if ($lookup == 'UNKNOWN') {
        _debug_echo("quova_ad_cache: $format: current visitor from UNKOWN locale.");
        $display = FALSE;
      }
      else {
        if ($target == 'city' && strpos($code, '*') !== FALSE) {
          // city with wildcard
          if (quova_ad_search($code, $lookup)) {
            _debug_echo("quova_ad_cache: $format:  current visitor from $lookup, matches $code.");
            $display = FALSE;
            break;
          }
        }
        _debug_echo("quova_ad_cache: $format: current visitor from $lookup not from $code.");
        $display = TRUE;
      }
      break;
    }
  }

  return $display;
}

/**
 * Convert $needle to regex, replacing "*" with "(.)*", then perform search.
 */
function quova_ad_search($needle, $haystack) {
  if (!strlen(trim($haystack))) return;
  $array = explode('*', $needle);
  $size = sizeof($array);
  $current = 0;
  foreach ($array as $string) {
    if (strlen(trim($string))) {
      if (++$current < $size) {
        $search .= $string ."(.)*";
      }
      else {
        $search .= $string;
      }
    }
    else {
      $current++;
    }
  }
  _debug_echo("quova_ad_cache: searching /$search/i.");
  if (preg_match("/$search/i", $haystack)) {
    return TRUE;
  }
  else {
    return FALSE;
  }
}

/**
 * Perform the appropriate GeoIP database lookup.
 */
function quova_ad_lookup($target) {
  switch ($target) {
    case 'country':
      $lookup = quova_ad_country_code();
      break;
    case 'region':
      $lookup = quova_ad_region_code();
      break;
    case 'city':
      $lookup = quova_ad_city_code();
      break;
  }
  return $lookup;
}

/**
 * Retrieve country code from IP address, cache for re-use.
 */
function quova_ad_country_code() {
  static $code = NULL;

  if (is_null($code)) {
    $hostname = ip_address();
    $code = geoip_country_code_by_name($hostname);
    if (!$code) {
      // do not translate, as we may call this without bootstrapping Drupal
      _debug_echo("quova_ad: no country code found for host from $hostname.");
      $code = 'UNKNOWN';
    }
  }
  return $code;
}

/**
 * Retrieve region code from IP address, cache for re-use.
 */
function quova_ad_region_code() {
  static $code = NULL;

  if (is_null($code)) {
    $hostname = ip_address();
    $result = geoip_record_by_name($hostname);
    if (!is_array($result) || empty($result)) {
      _debug_echo("quova_ad: no region code found for host from $hostname.");
      // do not translate, as we may call this without bootstrapping Drupal
      $code = 'UNKNOWN';
    }
    else {
      $code = $result['region'];
    }
  }
  return $code;
}

/**
 * Retrieve city code from IP address, cache for re-use.
 */
function quova_ad_city_code() {
  static $code = NULL;

  if (is_null($code)) {
    $hostname = ip_address();
    $result = geoip_record_by_name($hostname);
    if (!is_array($result) || empty($result)) {
      _debug_echo("quova_ad: no city found for host from $hostname.");
      // do not translate, as we may call this without bootstrapping Drupal
      $code = 'UNKNOWN';
    }
    else {
      $code = $result['city'];
    }
  }
  return $code;
}
